# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-09-16 16:22
from __future__ import unicode_literals

import datetime
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0002_connstr_length'),
    ]

    operations = [
        migrations.AddField(
            model_name='assignment',
            name='grace_period',
            field=models.DurationField(default=datetime.timedelta(0)),
        ),
        migrations.AddField(
            model_name='submission',
            name='in_grace_period',
            field=models.BooleanField(default=False),
        ),
    ]
